; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_INI:
	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)	
	
	
	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	out TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	out TWCR,temp

	; PULL-UP ein								
	sbi PORTC,4
	sbi PORTC,5


ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	out TWCR,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			   	
; START-ROUTINE	(MASTER TRANSMITTER)										   	
;																			   	
; INPUT:	temp1 <= ADRESSE SLAVE											   	
; OUT: 		FEHLER_CODE															
TWI_MASTER_TRANSMITTER_START_TEMP1:
	cbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB an	
	WDR
	; 	FEHLER lschen	
	clr FEHLER_CODE
	; START								
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	out TWCR, temp	
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08							; 0x08 START erfolgreich
	breq TWI_MASTER_TRANSMITTER_START_w
	cpi temp,  0x10							; 0x10 (repeated START)
	breq TWI_MASTER_TRANSMITTER_START_w
;-------------------------------------------------------------------------------
	; ERROR		
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	ldi temp,(ERROR_TWI_MT_START)
	mov FEHLER_CODE, temp
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1								; 1 x links
	out TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18
	breq TWI_MASTER_TRANSMITTER_START_OK
;-------------------------------------------------------------------------------
	; ERROR		
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	ldi temp,(ERROR_TWI_MT_SLA)
	mov FEHLER_CODE, temp
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_OK:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   	
; Ein BYTE zum SLAVE zu senden										   	   	   	
; 																			   	
; INPUT:	temp1 <= DATEN													   	
; OUT:		FEHLER_CODE															
; 																			   	
TWI_MASTER_TRANSMITTER_DATEN_TEMP1:
	; FEHLERPRFUNG						
	tst FEHLER_CODE							; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_DATEN_w		; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATEN_w:
	; 	Load DATA						
	out TWDR, temp1							
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	breq TWI_MASTER_TRANSMITTER_DATEN_ok
;-------------------------------------------------------------------------------
	; ERROR								
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	ldi temp,(ERROR_TWI_MT_DATA)
	mov FEHLER_CODE, temp
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATEN_ok:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			   	
; TWI/I2C START-ROUTINE														   	
;																			   	
; 	INPUT:	temp1 <= ADRESSE SLAVE											   	
;	OUT: 	FEHLER_CODE		   													
;																			   	
TWI_MASTER_RECEIVER_START_TEMP1:
	cbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB an	
	WDR
	; FEHLER lschen					
	clr FEHLER_CODE
	; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	out TWCR, temp	
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR 										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MASTER_RECEIVER_START_w
;-------------------------------------------------------------------------------
	; ERROR												
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	ldi temp,(ERROR_TWI_MR_START)
	mov FEHLER_CODE, temp
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_w:
	; Load SLA +"R" (R=1)								
	LSL  temp1	; 1 x links
	ori temp1, 0b00000001 
	out TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	breq TWI_MASTER_RECEIVER_ok
;-------------------------------------------------------------------------------
	; ERROR						
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	ldi temp,(ERROR_TWI_MR_SLA)
	mov FEHLER_CODE, temp
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_oK:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! BYTES empfangen werden sollen										
;																				
; 	OUT: 	temp1																
;	OUT: 	FEHLER_CODE		   													
;																				
TWI_MASTER_RECEIVER_BYTE_TEMP1:
	; Fehlerprfung										
	tst FEHLER_CODE
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_w
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_w:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_OK
;-------------------------------------------------------------------------------
	; ERROR
	clr temp1
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	ldi temp,(ERROR_TWI_MR_DATA)
	mov FEHLER_CODE, temp	
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_OK:
	; DATEN einlesen							
	in temp1, TWDR						
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! BYTE empfangen werden soll, oder wenn								
; das letzte! BYTE empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RECEIVER_LAST_BYTE_TEMP1:
	; Fehlerprfung										
	tst FEHLER_CODE
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_OK	; ERROR
;-------------------------------------------------------------------------------
	; ERROR
	clr temp1
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	ldi temp,(ERROR_TWI_MR_DATAEND)
	mov FEHLER_CODE, temp	
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_OK:
	; DATEN einlesen		
	in temp1, TWDR
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	; kleine Pause
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	out TWCR, temp	
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus	
	; kleine Pause
	; rcall wait_1ms
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   
; ERROR- Routine															   
; 																			   
TWI_ERROR_SCHWER:
	rcall TWI_STOP
	rcall wait_100ms	
	rcall TWI_STOP
	rcall wait_100ms	
	rcall TWI_STOP
	rcall wait_100ms	
	rcall TWI_AUS
	rcall wait_100ms	
	rcall TWI_INI
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	in temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; SCAn nach vorhandenen SLAVES												
TWI_SCAN:
	; LED aus			
	sbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN aus	
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus	
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	; NULL				
	STS(adr_TWI_SCAN_ADRESSE),NULL
	STS(adr_TWI_SCAN_TREFFER),NULL
	clr FEHLER_CODE
	; VOLL			
	STS(adr_TWI_SCAN_BEREICH+0),VOLL
	STS(adr_TWI_SCAN_BEREICH+1),VOLL
	STS(adr_TWI_SCAN_BEREICH+2),VOLL
	STS(adr_TWI_SCAN_BEREICH+3),VOLL
	STS(adr_TWI_SCAN_BEREICH+4),VOLL
	STS(adr_TWI_SCAN_BEREICH+5),VOLL
	STS(adr_TWI_SCAN_BEREICH+6),VOLL
	STS(adr_TWI_SCAN_BEREICH+7),VOLL
	STS(adr_TWI_SCAN_BEREICH+8),VOLL
	STS(adr_TWI_SCAN_BEREICH+9),VOLL
	STS(adr_TWI_SCAN_BEREICH+10),VOLL
	STS(adr_TWI_SCAN_BEREICH+11),VOLL
	STS(adr_TWI_SCAN_BEREICH+12),VOLL
	STS(adr_TWI_SCAN_BEREICH+13),VOLL
	STS(adr_TWI_SCAN_BEREICH+14),VOLL
	STS(adr_TWI_SCAN_BEREICH+15),VOLL
	STS(adr_TWI_SCAN_BEREICH+16),VOLL
	STS(adr_TWI_SCAN_BEREICH+17),VOLL
	STS(adr_TWI_SCAN_BEREICH+18),VOLL
	STS(adr_TWI_SCAN_BEREICH+19),VOLL
;-----------------------------------------------------------------------
	; ZEILE1	
	rcall LCD_CLEAR
;-----------------------------------------------------------------------
TWI_SCAN_s:
	; WDR							
	WDR
	; ANZEIGE						
	rcall LCD_ZEILE1
	ldi ZH, high(TEXT_SCAN*2)	; TEXT 
	ldi ZL,  low(TEXT_SCAN*2)
	rcall ANZEIGE_TEXT
	rcall LCD_SPACE
	LDS temp1,(adr_TWI_SCAN_ADRESSE)
	rcall LCD_AUSGABE_ZAHL_3_STELLEN_TEMP1		; inp: temp1
	; MASTER-START					
	LDS temp1,(adr_TWI_SCAN_ADRESSE)
	rcall TWI_MASTER_TRANSMITTER_START_TEMP1	; INP: temp1(SLAVE_ADRESSE) out: FEHLER_CODE
	; Auswertung					
	tst FEHLER_CODE
	brne TWI_SCAN_NO
	; Z initalisieren				
	LDS temp,(adr_TWI_SCAN_TREFFER)
	ldi ZL,low (adr_TWI_SCAN_BEREICH)
	ldi ZH,high(adr_TWI_SCAN_BEREICH)
	add ZL,temp
	adc ZH,NULL
	; ZEIGER +1						
	inc temp
	STS(adr_TWI_SCAN_TREFFER),temp
	; SPEICHERN nach Z				
	LDS temp1,(adr_TWI_SCAN_ADRESSE)
	ST z+,temp1	
	cbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN an   
TWI_SCAN_NO:
	; ADRESSE +1					
	LDS temp1,(adr_TWI_SCAN_ADRESSE)
	inc temp1
	andi temp1,0b01111111
	STS(adr_TWI_SCAN_ADRESSE),temp1
	tst temp1
	brne TWI_SCAN_s
	; STOP								
	rcall TWI_STOP
	clr FEHLER_CODE
	; AUSWERTUNG						
	LDS temp,(adr_TWI_SCAN_TREFFER)	
	tst temp
	brne TWI_SCAN_SLAVES_FOUND
	; ERROR (kein SLAVE gefunden)
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	rcall ANZEIGE_ERROR_ZEILE4
	ret
;-----------------------------------------------------------------------
TWI_SCAN_SLAVES_FOUND:	
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	; ANZEIGE						
	; ZEILE1	
	rcall LCD_CLEAR
	ldi ZH, high(TEXT_SCAN*2)	; TEXT 
	ldi ZL,  low(TEXT_SCAN*2)
	rcall ANZEIGE_TEXT
	rcall LCD_SPACE
	rcall ANZEIGE_OK_KURZ
	rcall LCD_SPACE
	LDS temp1,(adr_TWI_SCAN_TREFFER)	
	rcall LCD_AUSGABE_ZAHL_3_STELLEN_TEMP1		; inp: temp1
	rcall LCD_SPACE
	ldi temp,'S'
	rcall LCD_DATA_TEMP
	ldi temp,'L'
	rcall LCD_DATA_TEMP
	ldi temp,'A'
	rcall LCD_DATA_TEMP
	ldi temp,'V'
	rcall LCD_DATA_TEMP
	ldi temp,'E'
	rcall LCD_DATA_TEMP
	ldi temp,'S'
	rcall LCD_DATA_TEMP
	
	; ZEILE2	
	rcall LCD_ZEILE2
	LDS temp1,(adr_TWI_SCAN_BEREICH+0)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+1)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+2)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+3)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+4)
	rcall TWI_SCAN_DETAIL
	; ZEILE3	
	rcall LCD_ZEILE3
	LDS temp1,(adr_TWI_SCAN_BEREICH+5)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+6)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+7)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+8)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+9)
	rcall TWI_SCAN_DETAIL
	; ZEILE4	
	rcall LCD_ZEILE4
	LDS temp1,(adr_TWI_SCAN_BEREICH+10)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+11)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+12)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+13)
	rcall TWI_SCAN_DETAIL
	LDS temp1,(adr_TWI_SCAN_BEREICH+14)
	rcall TWI_SCAN_DETAIL
	
	; BEEP		
	rcall beep
	
	ret
;-----------------------------------------------------------------------
TWI_SCAN_DETAIL:
	cpi temp1,255
	brne TWI_SCAN_DETAIL_w
	rcall LCD_PUNKT
	rcall LCD_PUNKT
	rcall LCD_PUNKT
	rcall LCD_SPACE
	ret
TWI_SCAN_DETAIL_w:
	rcall LCD_AUSGABE_ZAHL_3_STELLEN_TEMP1	; inp: temp1
	rcall LCD_SPACE
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
